library IEEE;
use IEEE.STD_LOGIC_1164.all;
package DCOL_pack is
-- these constants are used to create VME Configuration ROM data
constant CMD_addr : std_logic_vector(31 downto 0) := x"00000000";
constant CSR_addr : std_logic_vector(31 downto 0) := x"00000004";
constant DCON_en_addr : std_logic_vector(31 downto 0) := x"00000008";
constant SIZE_addr : std_logic_vector(31 downto 0) := x"0000000c";
constant RP_addr : std_logic_vector(31 downto 0) := x"00000010";
constant WP_addr : std_logic_vector(31 downto 0) := x"00000014";
constant FPGA_addr : std_logic_vector(31 downto 0) := x"00000018";
constant FLASH_addr : std_logic_vector(31 downto 0) := x"0000001c";
constant PAGE_addr : std_logic_vector(31 downto 0) := x"00000020";
constant Tx_addr : std_logic_vector(31 downto 0) := x"00000028";
constant Rx_addr : std_logic_vector(31 downto 0) := x"0000002c";
constant FLASH_w_buf : std_logic_vector(31 downto 0) := x"00001000";
constant FLASH_r_buf : std_logic_vector(31 downto 0) := x"00001200";
constant MONITOR_buf : std_logic_vector(31 downto 0) := x"00001800";
constant Tx_buf : std_logic_vector(31 downto 0) := x"00002000";
constant Rx_buf : std_logic_vector(31 downto 0) := x"00002800";
constant PAGE_buf : std_logic_vector(31 downto 0) := x"00004000";
type vme_pad is record
	resetn:			std_logic;
	t_dtack:			std_logic;
	dtackn:			std_logic;
	berrn:			std_logic;
	iackn:			std_logic;
	iackinn:			std_logic;
	iackoutn:		std_logic;
	irq1n:			std_logic;
	irq2n:			std_logic;
	irq4n:			std_logic;
	irq7n:			std_logic;
	sysfailn:		std_logic;
	writen:			std_logic;
	asn:				std_logic;
	dsn: 				std_logic_vector(1 downto 0);
	am: 				std_logic_vector(5 downto 0);
	a: 				std_logic_vector(31 downto 0);
	gan: 				std_logic_vector(4 downto 0);
	gapn:				std_logic;
	d_dir:			std_logic;
	a_dir:			std_logic;
	oen:				std_logic;
	d: 				std_logic_vector(31 downto 0);
end record;
type mem_pad is record
	  mdq : STD_LOGIC_VECTOR (15 downto 0);
	  ma : STD_LOGIC_VECTOR (12 downto 0);
	  mba : STD_LOGIC_VECTOR (1 downto 0);
	  dm : STD_LOGIC_VECTOR (1 downto 0);
	  dqs : STD_LOGIC_VECTOR (1 downto 0);
	  we_n : STD_LOGIC;
	  ras_n : STD_LOGIC;
	  cas_n : STD_LOGIC;
	  cs_n : STD_LOGIC;
	  mclke : STD_LOGIC;
	  mclk : STD_LOGIC;
	  mclk_n : STD_LOGIC;   
end record;
COMPONENT vme_if
   generic (swap_byte : boolean := false; swap_lw : boolean := false);
	PORT(
		SW_BAR : IN std_logic_vector(4 downto 0);
		SW_use : IN std_logic;
		clk : IN std_logic;
		n_strobe : IN std_logic;
		mem_test : IN std_logic;
		ram_dav : IN std_logic;
		sel_mod : OUT std_logic;
		sel_reg : OUT std_logic;
		r_data : IN std_logic_vector(31 downto 0);    
		vme_pads : INOUT vme_pad;      
		vme_BLT : OUT std_logic;
		rfifo_afull : OUT std_logic;
		rd_ram : OUT std_logic;
		w_data : OUT std_logic_vector(31 downto 0);
		we_byte : OUT std_logic_vector(3 downto 0);
		sys_reset : OUT std_logic;
		addr : OUT std_logic_vector(22 downto 0)
		);
END COMPONENT;
COMPONENT DCON_rxin
	PORT(
		clk : IN std_logic;
		start_bit : IN std_logic;
		reset_rx : IN std_logic;
		chk_lock : IN std_logic;
		DCON_en : IN std_logic;          
		DCON_rx : IN std_logic;          
		rx_lock : OUT std_logic;          
		q : OUT std_logic
		);
END COMPONENT;
COMPONENT srl_fifo
   GENERIC (N: integer := 32);
	PORT(
		rst : IN std_logic;
		din : IN std_logic_vector(N-1 downto 0);
		we : IN std_logic;
		clk : IN std_logic;
		re : IN std_logic;          
		dav : OUT std_logic;          
		dout : OUT std_logic_vector(N-1 downto 0);
		rp : OUT std_logic_vector(3 downto 0)
		);
END COMPONENT;
COMPONENT delayN
    generic (N: positive := 2;
	          INIT : bit_vector := x"0000");
    port (
        c: in STD_LOGIC;
        d: in STD_LOGIC;
        ce: in STD_LOGIC;
        q: out STD_LOGIC
    );
end COMPONENT;
COMPONENT DCON_if
   generic (swap_byte : boolean := false);
	PORT(
		clk : IN std_logic;
		reset : IN std_logic;
		n_strobe : IN std_logic;
		start_bit : IN std_logic;
		trig_clk : IN std_logic;
		trig_d0 : IN std_logic;
		sel_reg : IN std_logic;
		addr : IN std_logic_vector(22 downto 0);
		w_data : IN std_logic_vector(31 downto 0);
		we_byte : IN std_logic_vector(3 downto 0);
    DCOLAD : in  STD_LOGIC_VECTOR (3 downto 0);
		DCON_en : IN std_logic_vector(12 downto 1);
		DCON_rx : IN std_logic_vector(12 downto 1);          
		ctrl_tx_data : OUT std_logic_vector(31 downto 0);
		ctrl_rx_data : OUT std_logic_vector(31 downto 0);
		DCON_tx : OUT std_logic_vector(12 downto 1);
		ctrl_tx_csr : OUT std_logic_vector(31 downto 0);
		ctrl_rx_csr : OUT std_logic_vector(11 downto 0);
    mon_cntr : OUT  STD_LOGIC_VECTOR (6 downto 0);
		buf_full : IN std_logic;
		DCON_err : OUT std_logic;
		DCON_data : OUT std_logic_vector(8 downto 0);
		DCON_addr : OUT std_logic_vector(5 downto 0);
		DCON_we : OUT std_logic_vector(2 downto 0)
		);
END COMPONENT;
COMPONENT flash
   generic (swap_byte : boolean := false);
	PORT(
		clk : IN std_logic;
		locked : IN std_logic;
		reset : IN std_logic;
		n_strobe : IN std_logic;
		sel_reg : IN std_logic;
		addr : IN std_logic_vector(22 downto 0);
		we_byte : IN std_logic_vector(3 downto 0);
		w_data : IN std_logic_vector(31 downto 0);
		MQ : IN std_logic;          
		CPLD_phase : IN std_logic;          
		flash_wdata : OUT std_logic_vector(31 downto 0);
		flash_rdata : OUT std_logic_vector(31 downto 0);
		flash_busy : OUT std_logic;
		reconf_b : OUT std_logic;
		MD : OUT std_logic;
		MW_n : OUT std_logic;
		MCS : OUT std_logic
		);
END COMPONENT;
COMPONENT trig_ctrl
	PORT(
		clk : IN std_logic;
		trig_in : IN std_logic;
		stand_alone : IN std_logic;
		second : IN std_logic;          
		DCON_en : IN std_logic_vector(12 downto 1);
		test_out : OUT std_logic_vector(12 downto 1);
		trig_out : OUT std_logic;
		start_bit : OUT std_logic
		);
END COMPONENT;
COMPONENT sorterA
	PORT(
		clk : IN std_logic;
		n_strobe : IN std_logic;
		reset : IN std_logic;
		skip : IN std_logic;
		one_ms : IN std_logic;
		DCON_data : IN std_logic_vector(8 downto 0);
		DCON_we : IN std_logic;
		DCON_addr : IN std_logic_vector(5 downto 0);          
		hit_data : OUT std_logic_vector(8 downto 0);
		hit_addr : OUT std_logic_vector(1 downto 0);
		hit_rqst : OUT std_logic;
		hit_end : OUT std_logic;
		hit_gnt : IN std_logic
		);
END COMPONENT;
COMPONENT sorterB
	PORT(
		clk : IN std_logic;
		n_strobe : IN std_logic;
		reset : IN std_logic;
		grp_dav : IN std_logic_vector(15 downto 0);
		grp_data : IN std_logic_vector(31 downto 0);
		grp_addr : IN std_logic_vector(5 downto 0);
		grp_we : IN std_logic;
		hit_gnt : IN std_logic;          
		grp_gnt : IN std_logic;
		grp_rqst : OUT std_logic;
		grp_num : OUT std_logic_vector(3 downto 0);
		hit_data : OUT std_logic_vector(7 downto 0);
		hit_rqst : OUT std_logic
		);
END COMPONENT;
COMPONENT sorterC
	PORT(
		clk : IN std_logic;
		n_strobe : IN std_logic;
		reset : IN std_logic;
		circular : IN std_logic;
		buf_full : IN std_logic;
		one_ms : IN std_logic;
		ch_data : IN std_logic_vector(31 downto 0);
		ch_addr : IN std_logic_vector(5 downto 0);
		ch_dav : IN std_logic_vector(11 downto 0);
		ch_we : IN std_logic;          
		ch_gnt : IN std_logic;          
		ch_rqst : OUT std_logic;
		ch_num : OUT std_logic_vector(3 downto 0);
		evt_data : OUT std_logic_vector(7 downto 0);
		evt_end : OUT std_logic;
		evt_we : OUT std_logic
		);
END COMPONENT;
COMPONENT sdram
	generic (sim : boolean := false);
	PORT(
		reset : IN std_logic;
		mclk0 : IN std_logic;
		mclk90 : IN std_logic;
		mclk180 : IN std_logic;
		mclk270 : IN std_logic;
		n_strobe : IN std_logic;
		locked : IN std_logic;
		mem_test : IN std_logic;
		run : IN std_logic;
		addr : IN std_logic_vector(22 downto 0);
		rd_ram : IN std_logic;
		ram_dav : IN std_logic;
		re_hit : IN std_logic;
		ddr_w_we : IN std_logic;
		ddr_read : IN std_logic;
		ddr_rqst : IN std_logic;
		ddr_length : IN std_logic_vector(3 downto 0);
		ddr_addr : IN std_logic_vector(22 downto 0);
		ddr_w_data : IN std_logic_vector(35 downto 0);    
		mem_pads : INOUT mem_pad;      
		mem_status : OUT std_logic_vector(15 downto 0);
		rfifo_data : OUT std_logic_vector(31 downto 0);
		ddr_gnt : OUT std_logic
		);
END COMPONENT;
COMPONENT ddr_w_queue
   generic (swap_byte : boolean := false);
	PORT(
		clk : IN std_logic;
		n_strobe : IN std_logic;
		reset : IN std_logic;
		run : IN std_logic;
		circular : IN std_logic;
		skip : IN std_logic;
		mem_test : IN std_logic;
		DCON_en : IN std_logic_vector(12 downto 1);
		hit_data : IN std_logic_vector(35 downto 0);
		hit_addr : IN std_logic_vector(5 downto 0);
		hit_rqst : IN std_logic_vector(3 downto 0);
		hit_end : IN std_logic_vector(3 downto 0);
		evt_end : IN std_logic;
		evt_we : IN std_logic;
		evt_data_in : IN std_logic_vector(7 downto 0);
		ddr_gnt : IN std_logic;
		sB_addr : IN std_logic_vector(3 downto 0);
		sB_rqst : IN std_logic;
		sC_addr : IN std_logic_vector(3 downto 0);
		sC_rqst : IN std_logic;
		w_data : IN std_logic_vector(31 downto 0);
		evt_wp : IN std_logic_vector(22 downto 0);
		page_reg : IN std_logic_vector(10 downto 0);
		addr : IN std_logic_vector(22 downto 0);
		we_byte : IN std_logic_vector(3 downto 0);
		rfifo_afull : IN std_logic;
		sel_mod : IN std_logic;
		rd_ram : IN std_logic;
		vme_BLT : IN std_logic;          
		hit_gnt : OUT std_logic_vector(3 downto 0);
		ddr_w_we : OUT std_logic;
		ddr_w_data : OUT std_logic_vector(35 downto 0);
		ddr_req : OUT std_logic;
		ddr_read : OUT std_logic;
		ddr_length : OUT std_logic_vector(3 downto 0);
		ddr_addr : OUT std_logic_vector(22 downto 0);
		sB_gnt : OUT std_logic;
		sB_g_dav : OUT std_logic_vector(15 downto 0);
		sC_gnt : OUT std_logic;
		sC_ch_dav : OUT std_logic_vector(11 downto 0);
		re_hit : OUT std_logic;
		we_sB : OUT std_logic;
		we_sC : OUT std_logic;
		sorter_wa : OUT std_logic_vector(5 downto 0);
		ram_dav : OUT std_logic;
		evt_size : OUT std_logic_vector(31 downto 0)
		);
END COMPONENT;
COMPONENT counters
	PORT(
		clk : IN std_logic;
		reset : IN std_logic;
		n_strobe : IN std_logic;
		addr : IN std_logic_vector(22 downto 0);
		trig_clk : IN std_logic;
		hit_gnt : IN std_logic_vector(3 downto 0);
		dcon_if_cnt : IN std_logic_vector(6 downto 0);
		DCON_err : IN std_logic;
		DCON_addr : IN std_logic_vector(5 downto 0);
		DCON_we : IN std_logic_vector(2 downto 0);          
		mon_data : OUT std_logic_vector(31 downto 0)
		);
END COMPONENT;
end DCOL_pack;
package body DCOL_pack is
end DCOL_pack;
