----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
--  
-- Create Date:    15:53:19 03/27/2006 
-- Design Name: 
-- Module Name:    atm_db_conf - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
-- two configuration data files can be stored:
-- first(default) one started at address 0. If it fails(done does not go high),
-- the second(backup) one which started at address 0x200000 will be used. 
-- Never update two files at the same time.
-- please note that in a byte of FLASH data, the MSB comes out the first,
-- so byte data in a mcs file must be rearranged before writing to the FLASH
-- Dependencies: 
--  
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--  
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
    
---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity DCOL_conf is
    generic (step : integer := 1;
	          sim : std_logic := '0');
    Port ( 
           reset_b : in  STD_LOGIC;
			  clk : in  STD_LOGIC; -- 25 MHz clock input
           MCLK : out  STD_LOGIC; -- FLASH clock
           MD : out  STD_LOGIC; -- FLASH data in
           MS_n : out  STD_LOGIC; -- FLASH chip select
           MW_n : out  STD_LOGIC; -- FLASH write protection
           MH_n : out  STD_LOGIC; -- FLASH hold
           INIT_b : in  STD_LOGIC;
           DONE : in  STD_LOGIC;
           CCLK : out  STD_LOGIC;
           PROG_b : out  STD_LOGIC;
           FPGA_ctrl : inout  STD_LOGIC_VECTOR(4 downto 0) -- FPGA interface
           );
end DCOL_conf;

architecture Behavioral of DCOL_conf is
signal cntr : std_logic_vector(26 downto 0) := "111" & x"fffff0";
signal clkdv : std_logic_vector(2 downto 0) := "000";
signal was_done : std_logic := '0';
signal en_MCLK : std_logic := '0';
signal en_CCLK : std_logic := '0';
attribute init : string;
attribute init of cntr : signal is "R";
begin
MW_n <= FPGA_ctrl(3);
FPGA_ctrl(3) <= 'Z';
FPGA_ctrl(2) <= 'Z';
FPGA_ctrl(0) <= 'Z';
MH_n <= '1';
MS_n <= not en_MCLK;
prog_b <= '0' when cntr(25) = '0' else 'Z';
FPGA_ctrl(1) <= cntr(26);
FPGA_ctrl(4) <= clkdv(1);
process(clk,reset_b)
begin
  if(reset_b = '0')then
    clkdv <= "000";
    en_MCLK <= '0';
    MCLK <= '0';
    CCLK <= '1';
    MD <= '0';
    was_done <= '0';
    cntr <= (others => '0');
    en_CCLK <= '0';
  elsif(clk'event and clk = '1')then
    clkdv(0) <= not clkdv(0);
    clkdv(1) <= clkdv(1) xor clkdv(0);
	 if(clkdv(0) = '1')then
	   if(clkdv(1) = '1')then
        en_MCLK <= INIT_b;
		end if;
      clkdv(2) <= not clkdv(1) and (en_MCLK or not(DONE and not en_CCLK) or not FPGA_ctrl(2));
	   MCLK <= not clkdv(2) and en_MCLK and cntr(25) and cntr(24);
      CCLK <= not clkdv(2) or not en_CCLK;
	   if(DONE = '0')then
	     if(clkdv(2) = '1')then
	       if(en_CCLK = '0' and (cntr(4 downto 0) = "00101" or cntr(4 downto 0) = "00110" 
		       or (cntr(26) = '1' and cntr(4 downto 0) = "01001")) and cntr(25 downto 24) = "11")then
	         MD <= '1';
          else
	         MD <= '0';
		    end if;	 
	     end if;
	   elsif(clkdv(1) = '1')then
	     MD <= FPGA_ctrl(0);
	   end if;
	   if(cntr(25) = '0')then
	     was_done <= '0';
	   elsif(DONE = '1')then
	     was_done <= '1';
	   end if;
--	 FPGA_ctrl(1) <= done and en_CCLK;
	   if(clkdv(2) = '1')then
	     if(cntr(25 downto 22) /= x"f" or FPGA_ctrl(2) = '0' or (DONE = '0' and cntr(26) = '0' and was_done = '0'))then
	       cntr(11 downto 0) <= cntr(11 downto 0) + 1;
		    if(cntr(11 downto 0) = x"fff" or sim = '1')then
	         cntr(25 downto 12) <= cntr(25 downto 12) + step;
			   if(cntr(25 downto 12) = "11111111111111")then
			     cntr(26) <= FPGA_ctrl(0) or FPGA_ctrl(2);
			   end if;
		    end if;
	     end if;
		  if(cntr(25 downto 24) /= "11" or cntr(23 downto 22) = "11")then
		    en_CCLK <= '0';
	     elsif(cntr(4 downto 1) = x"f")then
	       en_CCLK <= '1';
	     end if;
	   end if;	
    end if;
  end if;	 
end process;
end Behavioral;

