----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    15:39:40 06/29/2006 
-- Design Name: 
-- Module Name:    flash - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
-- flash buffer starts at 0x400
-- FPGA conf data must be in address ranges 0x600-0x7ff
-- once flash buffer filled with data, write number of bytes
-- to be sent to the FLASH to address 0x1, this starts the
-- 
----------------------------------------------------------------------------------
library IEEE,mylib;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use mylib.DCOL_pack.all;
---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity flash is
    generic (swap_byte : boolean := false);
    Port ( clk : in  STD_LOGIC;
           locked : in  STD_LOGIC;
           reset : in  STD_LOGIC;
           n_strobe : in  STD_LOGIC;
           sel_reg : in  STD_LOGIC;
           addr : in  STD_LOGIC_VECTOR (22 downto 0);
           we_byte : in  STD_LOGIC_VECTOR (3 downto 0);
           w_data : in  STD_LOGIC_VECTOR (31 downto 0);
           flash_wdata : out  STD_LOGIC_VECTOR (31 downto 0);
           flash_rdata : out  STD_LOGIC_VECTOR (31 downto 0);
           flash_busy : out  STD_LOGIC;
           reconf_b : out  STD_LOGIC;
           CPLD_phase : in  STD_LOGIC;
           MQ : in  STD_LOGIC;
           MD : out  STD_LOGIC;
           MW_n : out  STD_LOGIC;
           MCS : out  STD_LOGIC
			  );
end flash;

architecture Behavioral of flash is
signal strobe : std_logic := '0';
signal lock_cntr : std_logic_vector(7 downto 0) := (others => '0');
signal phase_sync : std_logic_vector(2 downto 0) := (others => '0');
signal cpld_strobe : std_logic_vector(3 downto 0) := (others => '0');
signal dia : std_logic_vector(0 downto 0) := (others => '0');
signal doa : std_logic_vector(0 downto 0) := (others => '0');
signal dib : std_logic_vector(31 downto 0) := (others => '0');
signal abcdef : std_logic_vector(31 downto 8) := (others => '0');
signal w_buf_rb : std_logic_vector(31 downto 0) := (others => '0');
signal addra : std_logic_vector(13 downto 0) := (others => '0');
signal addrap : std_logic_vector(13 downto 0) := (others => '0');
signal w_addra : std_logic_vector(13 downto 0) := (others => '0');
signal r_addra : std_logic_vector(13 downto 0) := (others => '0');
signal addrb : std_logic_vector(8 downto 0) := (others => '0');
signal byte_cnt : std_logic_vector(8 downto 0) := (others => '0');
signal en_FLASHp : std_logic := '0';
signal en_FLASH : std_logic := '0';
signal we_last_byte_dl : std_logic := '0';
signal we_last_byte : std_logic := '0';
signal en_FLASH_dl : std_logic_vector(1 downto 0) := "00";
signal en_FLASH_q : std_logic_vector(3 downto 0) := x"0";
signal ce_addra : std_logic := '0';
signal cpld_str : std_logic := '0';
signal md_p : std_logic := '0';
signal cmd : std_logic := '0';
signal rd : std_logic := '0';
signal wea : std_logic := '0';
signal web : std_logic := '0';
signal re_conf : std_logic := '0';
signal backup : std_logic := '0';
attribute INIT : string;
attribute INIT of re_conf,en_flash : signal is "0";
begin
MW_n <= '1';
reconf_b <= not re_conf;
flash_wdata <= w_buf_rb; 
we_last_byte <= we_byte(3) when swap_byte = false else we_byte(0);
process(clk)
begin
  if(clk'event and clk = '1')then
    strobe <= n_strobe;
    if(strobe = '1')then
			if(locked = '0')then
				lock_cntr <= (others => '0');
			elsif(lock_cntr(7) = '0')then
				lock_cntr <= lock_cntr + 1;
			end if;
			phase_sync <= phase_sync(1 downto 0) & CPLD_phase;
			if(lock_cntr(7) = '0' and phase_sync(2 downto 1) = "01")then
				cpld_strobe <= x"1";
			else
				cpld_strobe <= cpld_strobe(2 downto 0) & cpld_strobe(3);
			end if;
      flash_busy <= en_flash or en_flash_dl(1);
      if(addr = flash_w_buf(24 downto 2) and we_byte(0) = '1')then
				if(w_data(31 downto 24) = x"03")then
					rd <= '1';
				else
					rd <= '0';
				end if;
			end if;
			if(sel_reg = '1' and addr(3 downto 0) = FPGA_addr(5 downto 2))then
				If(we_byte(0) = '1')then
					abcdef(31 downto 24) <= w_data(31 downto 24);
				end if;
				If(we_byte(1) = '1')then
					abcdef(23 downto 16) <= w_data(23 downto 16);
				end if;
				If(we_byte(2) = '1')then
					abcdef(15 downto 8) <= w_data(15 downto 8);
				end if;
				If(we_byte(3) = '1')then
					backup <= w_data(0);
				end if;		  
				we_last_byte_dl <= we_last_byte;
			end if;
			If(we_last_byte_dl = '1')then
				If(abcdef = x"abcdef")then
					re_conf <= '1';
				end if;
				we_last_byte_dl <= '0';
			end if;
			if(en_FLASH = '0')then
				addra <= (others => '0');
				cmd <= '1';
			elsif(ce_addra = '1')then
				addra(11 downto 0) <= addra(11 downto 0) + 1;
				if(addra(4 downto 0) = "11111")then
					cmd <= '0';
				end if;
			end if;
			if(en_FLASH_dl(1) = '0')then
				addrap <= (others => '0');
			elsif(wea = '1')then
				addrap(11 downto 0) <= addrap(11 downto 0) + 1;
			end if;
-- FLASH signals
			if(sel_reg = '1' and addr(3 downto 0) = flash_addr(5 downto 2) and we_last_byte = '1')then
				en_FLASHp <= '1';
			elsif(en_FLASH = '1')then
				en_FLASHp <= '0';
			end if;
			if(en_FLASHp = '1' and cpld_strobe(0) = '1')then
				en_FLASH <= '1';
			elsif(addra(11 downto 3) = byte_cnt and addra(2 downto 0) = "111" and ce_addra = '1')then
				en_FLASH <= '0';
			end if;
			if(en_FLASH = '0')then
				ce_addra <= '0';
			else
				ce_addra <= cpld_strobe(3);
			end if;
			if(cpld_strobe(0) = '1')then
				en_FLASH_dl <= en_FLASH_dl(0) & en_FLASH;
				MCS <= en_FLASH_dl(0);
				MD <= md_p;
				md_p <= (re_conf and backup) or (not re_conf and doa(0));
			end if;  
			if(sel_reg = '1' and addr(3 downto 0) = flash_addr(5 downto 2))then
				if(we_byte(2) = '1')then
					byte_cnt(8) <= w_data(8);
				end if;
				if(we_byte(3) = '1')then
					byte_cnt(7 downto 0) <= w_data(7 downto 0);
				end if;
			end if;
			if(cpld_strobe(3) = '1')then
				en_FLASH_q(1) <= en_FLASH_q(0);
				en_FLASH_q(3) <= en_FLASH_q(2);
			end if;
			if(cpld_strobe(0) = '1')then
				wea <= en_FLASH_q(3) and en_FLASH_dl(1);
			else
				wea <= '0';
			end if;
		end if;
  end if;
end process;
dib(31 downto 24) <= w_data(31 downto 24) when we_byte(0) = '1' else w_buf_rb(31 downto 24);
dib(23 downto 16) <= w_data(23 downto 16) when we_byte(1) = '1' else w_buf_rb(23 downto 16);
dib(15 downto 8) <= w_data(15 downto 8) when we_byte(2) = '1' else w_buf_rb(15 downto 8);
dib(7 downto 0) <= w_data(7 downto 0) when we_byte(3) = '1' else w_buf_rb(7 downto 0);
web <= '1' when we_byte /= x"0" and addr(22 downto 7) = FLASH_w_buf(24 downto 9) else '0';
addrb <= "00" & addr(6 downto 0);
i_wbuf: ramb16_s1_s36
     port map(
         dia => "0", 
         addra => w_addra,
         clka => clk, 
         ssra => '0', 
         ena => strobe, 
         wea => '0',
         dib => dib,
         dipb => x"0",
         addrb => addrb,
         clkb => clk,
         ssrb => '0',
         enb => strobe,
         web => web,
			dob => w_buf_rb,
			doa => doa);
w_addra(13 downto 5) <= addra(13 downto 5);
w_addra(4 downto 0) <= not addra(4 downto 3) & addra(2 downto 0) when cmd = '0' else not addra(4 downto 0);
i_rbuf: ramb16_s1_s36
     port map(
         dia => dia, 
         addra => r_addra,
         clka => clk, 
         ssra => '0', 
         ena => strobe, 
         wea => wea,
         dib => x"00000000",
         dipb => x"0",
         addrb => addrb,
         clkb => clk,
         ssrb => '0',
         enb => strobe,
         web => '0',
         dob => flash_rdata);
dia(0) <= MQ;
r_addra(13 downto 5) <= addrap(13 downto 5);
r_addra(4 downto 0) <= not addrap(4 downto 3) & addrap(2 downto 0) when rd = '1' else not addrap(4 downto 0);
i_SRLC16a : SRL16E
   port map (
      Q => en_FLASH_q(0),
      A0 => '1',
      A1 => '1',
      A2 => rd,
      A3 => rd,
		CE => cpld_str,
      CLK => CLK,
      D => en_FLASH_dl(0)
   );
i_SRL16b : SRL16E
   port map (
      Q => en_FLASH_q(2),
      A0 => '1',
      A1 => '1',
      A2 => rd,
      A3 => rd,
		CE => cpld_str,
      CLK => CLK,
      D => en_FLASH_q(1)
   );
cpld_str <= cpld_strobe(3) and strobe;

end Behavioral;

